@@ -109,7 +109,8 @@ public:
   PPTraceConsumer(SmallSet<std::string, 4> &Ignore,
                   std::vector<CallbackCall> &CallbackCalls, Preprocessor &PP) {
     // PP takes ownership.
-    PP.addPPCallbacks(new PPCallbacksTracker(Ignore, CallbackCalls, PP));
+    PP.addPPCallbacks(llvm::make_unique<PPCallbacksTracker>(Ignore,
+                                                            CallbackCalls, PP));
   }
 };
 
@@ -120,9 +121,10 @@ public:
       : Ignore(Ignore), CallbackCalls(CallbackCalls) {}
 
 protected:
-  virtual clang::ASTConsumer *CreateASTConsumer(CompilerInstance &CI,
-                                                StringRef InFile) {
-    return new PPTraceConsumer(Ignore, CallbackCalls, CI.getPreprocessor());
+  std::unique_ptr<clang::ASTConsumer>
+  CreateASTConsumer(CompilerInstance &CI, StringRef InFile) override {
+    return llvm::make_unique<PPTraceConsumer>(Ignore, CallbackCalls,
+                                              CI.getPreprocessor());
   }
 
 private:
@@ -199,8 +201,8 @@ int main(int Argc, const char **Argv) {
 
   // Create the tool and run the compilation.
   ClangTool Tool(*Compilations, SourcePaths);
-  int HadErrors =
-      Tool.run(new PPTraceFrontendActionFactory(Ignore, CallbackCalls));
+  PPTraceFrontendActionFactory Factory(Ignore, CallbackCalls);
+  int HadErrors = Tool.run(&Factory);
 
   // If we had errors, exit early.
   if (HadErrors)
@@ -211,12 +213,11 @@ int main(int Argc, const char **Argv) {
     HadErrors = outputPPTrace(CallbackCalls, llvm::outs());
   } else {
     // Set up output file.
-    std::string Error;
-    llvm::tool_output_file Out(OutputFileName.c_str(), Error,
-                               llvm::sys::fs::F_Text);
-    if (!Error.empty()) {
+    std::error_code EC;
+    llvm::tool_output_file Out(OutputFileName, EC, llvm::sys::fs::F_Text);
+    if (EC) {
       llvm::errs() << "pp-trace: error creating " << OutputFileName << ":"
-                   << Error << "\n";
+                   << EC.message() << "\n";
       return 1;
     }
 
